"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.request = void 0;
const http = __importStar(require("http"));
const createRequest = async ({ requestOptions, requestBody, hijack, callback, timeout }) => {
    const req = http.request(requestOptions);
    const cancelTimeout = (() => {
        if (!timeout) {
            return () => undefined;
        }
        const timer = setTimeout(() => {
            req.destroy();
        }, timeout);
        return () => clearTimeout(timer);
    })();
    if (requestBody) {
        req.write(requestBody);
    }
    if (hijack) {
        cancelTimeout();
        req.on("upgrade", function (res, sock, head) {
            console.log(sock);
            callback === null || callback === void 0 ? void 0 : callback(sock);
        });
    }
    else {
        req.end();
    }
    return new Promise((resolve, reject) => {
        req.on("connect", () => {
            cancelTimeout();
        });
        req.on("close", () => {
            cancelTimeout();
        });
        req.on("response", res => {
            cancelTimeout();
            resolve(res);
        });
        req.on("error", error => {
            cancelTimeout();
            reject(error);
        });
    });
};
const createResponse = async ({ res, isStream, callback }) => {
    const chunks = [];
    res.on("data", chunk => {
        chunks.push(chunk);
    });
    return new Promise((resolve, reject) => {
        res.on("error", error => {
            reject(error);
        });
        if (isStream) {
            if (!callback) {
                throw new Error("Please use callback");
            }
            callback(res);
        }
        else {
            res.on("end", () => {
                const buffer = Buffer.concat(chunks);
                if (res.statusCode && 400 <= res.statusCode) {
                    const errorMessage = `[HTTP Code: ${res.statusCode}] ${res.statusMessage}.`;
                    const error = new Error(errorMessage);
                    reject(error);
                }
                const text = buffer.toString();
                try {
                    const json = JSON.parse(text);
                    resolve(json);
                }
                catch (error) {
                    resolve(text);
                }
            });
        }
    });
};
const request = async (args) => {
    const requestBody = !!args.requestBody ? JSON.stringify(args.requestBody) : undefined;
    const requestHeaders = {
        ...args.headers,
    };
    if (requestBody) {
        requestHeaders["Content-Length"] = Buffer.byteLength(requestBody, "utf-8");
    }
    if (args.hijack) {
        requestHeaders["Transfer-Encoding"] = "chunked";
        requestHeaders.Connection = "Upgrade";
        requestHeaders.Upgrade = "tcp";
    }
    const requestOptions = {
        method: args.method,
        socketPath: args.socketPath,
        path: args.path,
        headers: requestHeaders,
    };
    try {
        const res = await createRequest({
            requestOptions,
            requestBody,
            hijack: args.hijack,
            callback: args.onHijackRequest,
            timeout: args.timeout,
        });
        return await createResponse({ res, callback: args.onResponse, isStream: args.isStream });
    }
    catch (error) {
        if (error instanceof Error) {
            const errorMessage = [error.message, `${requestOptions.method}:${requestOptions.path}`].join("\n");
            throw new Error(errorMessage);
        }
        else {
            throw new Error("Unknown Error");
        }
    }
};
exports.request = request;
