import * as Formatter from "@himenon/openapi-parameter-formatter";
import * as http from "./http";
export const generateQueryString = (queryParameters, additionalString) => {
    if (!queryParameters) {
        return undefined;
    }
    const queries = Object.entries(queryParameters).reduce((queryStringList, [key, item]) => {
        if (!item.value) {
            return queryStringList;
        }
        if (!item.style) {
            return queryStringList.concat(`${key}=${item.value}`);
        }
        const result = Formatter.QueryParameter.generate(key, item);
        if (result) {
            return queryStringList.concat(result);
        }
        return queryStringList;
    }, []);
    if (additionalString) {
        return [...queries, additionalString].join("&");
    }
    return queries.join("&");
};
export const create = (params) => {
    const { socketPath } = params;
    const apiClientImpl = {
        request: async (httpMethod, url, headers, requestBody, queryParameters, requestOptions) => {
            // Docker's openapi Schema incorrectly uses filters as query parameter
            const { filters, ...omitedQueryParamsteres } = queryParameters || {};
            const query = generateQueryString(omitedQueryParamsteres, filters === null || filters === void 0 ? void 0 : filters.value);
            const requestUrl = query ? url + "?" + encodeURI(query) : url;
            const requestHeaders = {
                ...headers,
            };
            const response = await http.request({
                socketPath: socketPath,
                method: httpMethod,
                headers: requestHeaders,
                path: requestUrl,
                requestBody: requestBody,
                hijack: requestOptions === null || requestOptions === void 0 ? void 0 : requestOptions.hijack,
                isStream: requestOptions === null || requestOptions === void 0 ? void 0 : requestOptions.isStream,
                onHijackRequest: requestOptions === null || requestOptions === void 0 ? void 0 : requestOptions.onHijackRequest,
                onResponse: requestOptions === null || requestOptions === void 0 ? void 0 : requestOptions.onResponse,
            });
            return response;
        },
    };
    return apiClientImpl;
};
